
#include <stdio.h>
#include <stdlib.h>
#include <linux/input.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include "SC_PlugIn.h"

#define BUFFER_SIZE 32

int powermate_fd = -1;

// InterfaceTable contains pointers to functions in the host (server).
static InterfaceTable *ft;

// declare struct to hold unit generator state
struct Powermate : public Unit
{
  int value;
  int pressed;
  int last_position;
};

// declare unit generator functions 
extern "C"
{
  void load(InterfaceTable *inTable);
  void Powermate_next_k(Powermate *unit, int inNumSamples);
  void Powermate_Ctor(Powermate* unit);
};

#define NUM_VALID_PREFIXES 2

static const char *valid_prefix[NUM_VALID_PREFIXES] = {
  "Griffin PowerMate",
  "Griffin SoundKnob"
};

#define NUM_EVENT_DEVICES 16

int open_powermate(const char *dev, int mode)
{
  int fd = open(dev, mode);
  int i;
  char name[255];

  if(fd < 0){
    fprintf(stderr, "Unable to open \"%s\": %s\n", dev, strerror(errno));
    return -1;
  }

  if(ioctl(fd, EVIOCGNAME(sizeof(name)), name) < 0){
    fprintf(stderr, "\"%s\": EVIOCGNAME failed: %s\n", dev, strerror(errno));
    close(fd);
    return -1;
  }

  // it's the correct device if the prefix matches what we expect it to be:
  for(i=0; i<NUM_VALID_PREFIXES; i++)
    if(!strncasecmp(name, valid_prefix[i], strlen(valid_prefix[i])))
      return fd;

  close(fd);
  return -1;
}

int find_powermate(int mode)
{
  char devname[256];
  int i, r;

  for(i=0; i<NUM_EVENT_DEVICES; i++){
    sprintf(devname, "/dev/input/event%d", i);
    r = open_powermate(devname, mode);
    if(r >= 0)
      return r;
  }

  return -1;
}


void Powermate_next_k(Powermate *unit, int inNumSamples) {
  float result;
  float *out = OUT(0);
  int i;
  
  int abs_offset = 0;

  struct input_event ibuffer[BUFFER_SIZE];
  int r, events;

  if (powermate_fd > -1) {
    r = read(powermate_fd, ibuffer, sizeof(struct input_event) * BUFFER_SIZE);
    if (r > 0) {
      struct input_event *ev = &ibuffer[i];
      events = r / sizeof(struct input_event);
      for(i = 0; i < events; i++) {
	switch(ev->type) {
	case EV_REL:
	  if(ev->code == REL_DIAL) {
	    unit->value += (int) ev->value;
	  }
	  break;
	case EV_KEY:
	  if(ev->code == BTN_0) {
	    unit->pressed = ev->value;
	  }
	  break;
	}
      }
    }  
  }
  
  for (i=0; i < inNumSamples; ++i) {
    out[i] = unit->value;
  }
}

// unit generator initialisation
void Powermate_Ctor(Powermate* unit)
{
  // 1. Set calculation function -- always run at control rate
  SETCALC(Powermate_next_k);
  
  // 2. initialise
  unit->last_position = 0;
  unit->value = 0;
  unit->pressed = 0;

  // 3. Calculate one sample of output.
  // (why do this?)
  Powermate_next_k(unit, 1);
}

////////////////////////////////////////////////////////////////////

// the load function is called by the host when the plug-in is loaded
void load(InterfaceTable *inTable) {
  powermate_fd = find_powermate(O_RDONLY|O_NONBLOCK);

  if(powermate_fd < 0){
    fprintf(stderr, "Unable to locate powermate\n");
  }
  else {
    printf("opened powermate\n");
  }

  ft = inTable;
  DefineSimpleUnit(Powermate);
}

////////////////////////////////////////////////////////////////////
